﻿using System;
using Silanis.ESL.SDK;
using Silanis.ESL.SDK.Builder;
using System.Collections.Generic;
using System.Diagnostics;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.IO;
using Newtonsoft.Json.Linq;

namespace FeatureGuideCharp
{
    class OptionalSignatureREST
    {
        private string API_KEY = "your_api_key";
        private string API_URL = "https://sandbox.esignlive.com/api";
        private string FILE_PATH = "file_path_for_basic_usage";
        private string DOCUMENT_EXTRACTION_FILE_PATH = "file_path_for_document_extraction";
        private string TEXT_TAGS_FILE_PATH = "file_path_for_text_tags_extraction";


        public void BasicUsage()
        {
            
            // Prepare the client
            HttpClient httpClient = new HttpClient();
            httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", API_KEY);
            httpClient.DefaultRequestHeaders.Add("Accept", "application/json");

            // Build the request
            StringBuilder sb = new StringBuilder();

            sb.AppendLine("           {                                                         ");
            sb.AppendLine(" \"documents\": [                                                    ");
            sb.AppendLine("   {                                                                 ");
            sb.AppendLine("     \"approvals\": [                                                ");
            sb.AppendLine("       {                                                             ");
            sb.AppendLine("         \"fields\": [                                               ");
            sb.AppendLine("           {                                                         ");
            sb.AppendLine("             \"height\": 50,                                         ");
            sb.AppendLine("             \"left\": 100,                                          ");
            sb.AppendLine("             \"page\": 0,                                            ");
            sb.AppendLine("             \"subtype\": \"FULLNAME\",                              ");
            sb.AppendLine("             \"top\": 100,                                           ");
            sb.AppendLine("             \"type\": \"SIGNATURE\",                                ");
            sb.AppendLine("             \"width\": 200                                          ");
            sb.AppendLine("           }                                                         ");
            sb.AppendLine("         ],                                                          ");
            sb.AppendLine("         \"id\":\"signature1\",                                      ");
            sb.AppendLine("         \"role\": \"Role1\"                                         ");
            sb.AppendLine("       },                                                            ");
            sb.AppendLine("       {                                                             ");
            sb.AppendLine("         \"fields\": [                                               ");
            sb.AppendLine("           {                                                         ");
            sb.AppendLine("             \"height\": 50,                                         ");
            sb.AppendLine("             \"left\": 100,                                          ");
            sb.AppendLine("             \"page\": 0,                                            ");
            sb.AppendLine("             \"subtype\": \"FULLNAME\",                              ");
            sb.AppendLine("             \"top\": 300,                                           ");
            sb.AppendLine("             \"type\": \"SIGNATURE\",                                ");
            sb.AppendLine("             \"width\": 200                                          ");
            sb.AppendLine("           }                                                         ");
            sb.AppendLine("         ],                                                          ");
            sb.AppendLine("         \"id\":\"signature2\",                                      ");
            sb.AppendLine("         \"role\": \"Role1\",                                        ");
            sb.AppendLine("         \"optional\": true                                          ");
            sb.AppendLine("       }                                                             ");
            sb.AppendLine("     ],                                                              ");
            sb.AppendLine("     \"name\": \"Test Document\"                                     ");
            sb.AppendLine("   }                                                                 ");
            sb.AppendLine(" ],                                                                  ");
            sb.AppendLine(" \"name\": \"Optional Signature Basic Usage REST\",                  ");
            sb.AppendLine(" \"roles\": [                                                        ");
            sb.AppendLine("   {                                                                 ");
            sb.AppendLine("     \"id\": \"Role1\",                                              ");
            sb.AppendLine("     \"signers\": [                                                  ");
            sb.AppendLine("       {                                                             ");
            sb.AppendLine("         \"email\": \"signer1@mailinator.com\",                      ");
            sb.AppendLine("         \"firstName\": \"1.firstname\",                             ");
            sb.AppendLine("         \"lastName\": \"1.lastname\",                               ");
            sb.AppendLine("         \"company\": \"OneSpan Sign\"                               ");
            sb.AppendLine("       }                                                             ");
            sb.AppendLine("     ]                                                               ");
            sb.AppendLine("   }                                                                 ");
            sb.AppendLine(" ],                                                                  ");
            sb.AppendLine(" \"type\": \"PACKAGE\",                                              ");
            sb.AppendLine(" \"status\": \"SENT\"                                                ");
            sb.AppendLine(" }                                                                   ");


            StringContent requestContent = new StringContent(sb.ToString(), Encoding.UTF8, "application/json");

            // File upload
            byte[] fileByteArray = File.ReadAllBytes(FILE_PATH);
            ByteArrayContent byteArrayContent = new ByteArrayContent(fileByteArray);
            byteArrayContent.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data");
            byteArrayContent.Headers.ContentDisposition.Name = "\"file\"";
            byteArrayContent.Headers.ContentDisposition.FileName = "\"Document 1\"";
            byteArrayContent.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

            MultipartFormDataContent form = new MultipartFormDataContent();
            form.Add(byteArrayContent, "\"file\"", "\"Document 1\"");
            form.Add(requestContent, "\"payload\"");


            // Create/Send Package
            var response = httpClient.PostAsync(new Uri(API_URL) + "/packages/", form).Result;
            if (response.StatusCode == System.Net.HttpStatusCode.OK)
            {
                var stringResult = response.Content.ReadAsStringAsync().Result;
                var jsonResult = JObject.Parse(stringResult);
                JToken packageId = jsonResult["id"];
                Debug.WriteLine("Package Id: " + packageId);
            }
            else {
                Debug.WriteLine("Error Message: " + response.Content.ReadAsStringAsync().Result);
            }

        }



        public void WithDocumentExtraction()
        {

            // Prepare the client
            HttpClient httpClient = new HttpClient();
            httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", API_KEY);
            httpClient.DefaultRequestHeaders.Add("Accept", "application/json");

            // Build the request
            StringBuilder sb = new StringBuilder();


            sb.AppendLine("            {                                                                ");
            sb.AppendLine("  \"documents\": [                                                           ");
            sb.AppendLine("    {                                                                        ");
            sb.AppendLine("      \"extract\": true,                                                     ");
            sb.AppendLine("      \"name\": \"Test Document\"                                            ");
            sb.AppendLine("    }                                                                        ");
            sb.AppendLine("  ],                                                                         ");
            sb.AppendLine("  \"name\": \"Optional Signature with Document Extraction REST\",            ");
            sb.AppendLine("  \"roles\": [                                                               ");
            sb.AppendLine("    {                                                                        ");
            sb.AppendLine("      \"id\": \"Role1\",                                                     ");
            sb.AppendLine("      \"signers\": [                                                         ");
            sb.AppendLine("        {                                                                    ");
            sb.AppendLine("          \"email\": \"signer1@mailinator.com\",                             ");
            sb.AppendLine("          \"firstName\": \"1.firstname\",                                    ");
            sb.AppendLine("          \"lastName\": \"1.lastname\",                                      ");
            sb.AppendLine("          \"company\": \"OneSpan Sign\"                                      ");
            sb.AppendLine("        }                                                                    ");
            sb.AppendLine("      ]                                                                      ");
            sb.AppendLine("    }                                                                        ");
            sb.AppendLine("  ],                                                                         ");
            sb.AppendLine("  \"type\": \"PACKAGE\",                                                     ");
            sb.AppendLine("  \"status\": \"SENT\"                                                       ");
            sb.AppendLine("}                                                                            ");



            StringContent requestContent = new StringContent(sb.ToString(), Encoding.UTF8, "application/json");

            // File upload
            byte[] fileByteArray = File.ReadAllBytes(DOCUMENT_EXTRACTION_FILE_PATH);
            ByteArrayContent byteArrayContent = new ByteArrayContent(fileByteArray);
            byteArrayContent.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data");
            byteArrayContent.Headers.ContentDisposition.Name = "\"file\"";
            byteArrayContent.Headers.ContentDisposition.FileName = "\"Document 1\"";
            byteArrayContent.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

            MultipartFormDataContent form = new MultipartFormDataContent();
            form.Add(byteArrayContent, "\"file\"", "\"Document 1\"");
            form.Add(requestContent, "\"payload\"");


            // Create/Send Package
            var response = httpClient.PostAsync(new Uri(API_URL) + "/packages/", form).Result;
            if (response.StatusCode == System.Net.HttpStatusCode.OK)
            {
                var stringResult = response.Content.ReadAsStringAsync().Result;
                var jsonResult = JObject.Parse(stringResult);
                JToken packageId = jsonResult["id"];
                Debug.WriteLine("Package Id: " + packageId);
            }
            else
            {
                Debug.WriteLine("Error Message: " + response.Content.ReadAsStringAsync().Result);
            }

        }


        public void WithTextTagsExtraction()
        {

            // Prepare the client
            HttpClient httpClient = new HttpClient();
            httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", API_KEY);
            httpClient.DefaultRequestHeaders.Add("Accept", "application/json");

            // Build the request
            StringBuilder sb = new StringBuilder();


            sb.AppendLine("            {                                                               ");
            sb.AppendLine("  \"documents\": [                                                          ");
            sb.AppendLine("                {                                                           ");
            sb.AppendLine("      \"extract\": true,                                                    ");
            sb.AppendLine("      \"extractionTypes\":[                                                 ");
            sb.AppendLine("         \"TEXT_TAGS\"                                                      ");
            sb.AppendLine("      ],                                                                    ");
            sb.AppendLine("      \"name\": \"Test Document\"                                           ");
            sb.AppendLine("    }                                                                       ");
            sb.AppendLine("  ],                                                                        ");
            sb.AppendLine("  \"name\": \"Optional Signature with Text Tags Extraction REST\",          ");
            sb.AppendLine("              \"roles\": [                                                  ");
            sb.AppendLine("                {                                                           ");
            sb.AppendLine("      \"id\": \"Role1\",                                                    ");
            sb.AppendLine("      \"signers\": [                                                        ");
            sb.AppendLine("        {                                                                   ");
            sb.AppendLine("          \"email\": \"signer1@mailinator.com\",                            ");
            sb.AppendLine("          \"firstName\": \"1.firstname\",                                   ");
            sb.AppendLine("          \"lastName\": \"1.lastname\",                                     ");
            sb.AppendLine("          \"company\": \"OneSpan Sign\"                                     ");
            sb.AppendLine("        }                                                                   ");
            sb.AppendLine("      ]                                                                     ");
            sb.AppendLine("    }                                                                       ");
            sb.AppendLine("  ],                                                                        ");
            sb.AppendLine("  \"type\": \"PACKAGE\",                                                    ");
            sb.AppendLine("              \"status\": \"SENT\"                                          ");
            sb.AppendLine("            }                                                               ");


            StringContent requestContent = new StringContent(sb.ToString(), Encoding.UTF8, "application/json");

            // File upload
            byte[] fileByteArray = File.ReadAllBytes(TEXT_TAGS_FILE_PATH);
            ByteArrayContent byteArrayContent = new ByteArrayContent(fileByteArray);
            byteArrayContent.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data");
            byteArrayContent.Headers.ContentDisposition.Name = "\"file\"";
            byteArrayContent.Headers.ContentDisposition.FileName = "\"Document 1\"";
            byteArrayContent.Headers.ContentType = new MediaTypeHeaderValue("application/pdf");

            MultipartFormDataContent form = new MultipartFormDataContent();
            form.Add(byteArrayContent, "\"file\"", "\"Document 1\"");
            form.Add(requestContent, "\"payload\"");


            // Create/Send Package
            var response = httpClient.PostAsync(new Uri(API_URL) + "/packages/", form).Result;
            if (response.StatusCode == System.Net.HttpStatusCode.OK)
            {
                var stringResult = response.Content.ReadAsStringAsync().Result;
                var jsonResult = JObject.Parse(stringResult);
                JToken packageId = jsonResult["id"];
                Debug.WriteLine("Package Id: " + packageId);
            }
            else
            {
                Debug.WriteLine("Error Message: " + response.Content.ReadAsStringAsync().Result);
            }

        }


    }
}
